# coding=utf-8
# pystray
# Copyright (C) 2015-2024 Moses Palmér
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import contextlib
import locale
import sys
import threading

import pynput.keyboard

from six.moves import input

from . import EventTest


class KeyboardControllerTest(EventTest):
    NOTIFICATION = (
        'This test case is non-interactive, so you must not use the '
        'keyboard.\n'
        'You must, however, keep this window focused.')
    CONTROLLER_CLASS = pynput.keyboard.Controller
    LISTENER_CLASS = pynput.keyboard.Listener

    def decode(self, string):
        """Decodes a string read from ``stdin``.

        :param str string: The string to decode.
        """
        if sys.version_info.major >= 3:
            yield string
        else:
            for encoding in (
                    'utf-8',
                    locale.getpreferredencoding(),
                    sys.stdin.encoding):
                if encoding:
                    try:
                        yield string.decode(encoding)
                    except:
                        pass

    @contextlib.contextmanager
    def capture(self):
        """Captures a string in a code block.

        :returns: a callable which returns the actual data read
        """
        data = []

        #: The thread body that reads a line from stdin and appends it to data
        def reader():
            while reader.running:
                data.append(sys.stdin.readline()[:-1])
        reader.running = True

        # Start the thread
        thread = threading.Thread(target=reader)
        thread.start()

        # Run the code block
        try:
            yield lambda: tuple(self.decode(''.join(data)))

        finally:
            # Send a newline to let sys.stdin.readline return in reader
            reader.running = False
            self.controller.tap(pynput.keyboard.Key.enter)
            thread.join()

    def assert_input(self, failure_message, expected):
        """Asserts that a specific text is generated when typing.

        :param str failure_message: The message to display upon failure.

        :param text: The text to type and expect.
        """
        with self.capture() as collect:
            self.controller.type(expected)

        self.assertIn(expected, collect(), failure_message)

    def test_keys(self):
        """Asserts that all keys defined for the base keyboard interface are
        defined for the current platform"""
        from pynput.keyboard._base import Key
        for key in Key:
            self.assertTrue(
                hasattr(pynput.keyboard.Key, key.name),
                '%s is not defined for the current platform' % key.name)

    def test_press_invalid(self):
        with self.assertRaises(self.controller.InvalidKeyException):
            self.controller.press(True)

    def test_release_invalid(self):
        with self.assertRaises(self.controller.InvalidKeyException):
            self.controller.release(True)

    def test_press_release(self):
        """Asserts that a press followed by a release generates a typed string
        for an ascii character"""
        with self.capture() as collect:
            self.controller.tap(pynput.keyboard.Key.space)

        self.assertIn(
            u' ',
            collect(),
            'Failed to press and release space')

    def test_touch(self):
        """Asserts that the touch shortcut behaves as expected"""
        with self.capture() as collect:
            self.controller.touch(pynput.keyboard.Key.space, True)
            self.controller.touch(pynput.keyboard.Key.space, False)

        self.assertIn(
            u' ',
            collect(),
            'Failed to press and release space')

    def test_touch_dead(self):
        """Asserts that pressing dead keys generate combined characters"""
        with self.capture() as collect:
            dead = pynput.keyboard.KeyCode.from_dead(u'~')
            self.controller.tap(dead)
            self.controller.tap(u'a')

        self.assertIn(
            u'ã',
            collect(),
            'Failed to apply dead key')

    def test_touch_dead_space(self):
        """Asserts that pressing dead keys followed by space yields the
        non-dead version"""
        with self.capture() as collect:
            dead = pynput.keyboard.KeyCode.from_dead(u'~')
            self.controller.tap(dead)
            self.controller.tap(pynput.keyboard.Key.space)

        self.assertIn(
            u'~',
            collect(),
            'Failed to apply dead key')

    def test_touch_dead_twice(self):
        """Asserts that pressing dead keys twice yields the non-dead version"""
        with self.capture() as collect:
            dead = pynput.keyboard.KeyCode.from_dead(u'~')
            self.controller.tap(dead)
            self.controller.tap(dead)

        self.assertIn(
            u'~',
            collect(),
            'Failed to apply dead key')

    def test_alt_pressed(self):
        """Asserts that alt_pressed works"""
        # We do not test alt_r, since that does not necessarily exist on the
        # keyboard
        for key in (
                pynput.keyboard.Key.alt,
                pynput.keyboard.Key.alt_l):
            self.controller.press(key)
            self.assertTrue(
                self.controller.alt_pressed,
                'alt_pressed was not set with %s down' % key.name)
            self.controller.release(key)
            self.assertFalse(
                self.controller.alt_pressed,
                'alt_pressed was incorrectly set')

    def test_ctrl_pressed(self):
        """Asserts that ctrl_pressed works"""
        for key in (
                pynput.keyboard.Key.ctrl,
                pynput.keyboard.Key.ctrl_l,
                pynput.keyboard.Key.ctrl_r):
            self.controller.press(key)
            self.assertTrue(
                self.controller.ctrl_pressed,
                'ctrl_pressed was not set with %s down' % key.name)
            self.controller.release(key)
            self.assertFalse(
                self.controller.ctrl_pressed,
                'ctrl_pressed was incorrectly set')

    def test_shift_pressed(self):
        """Asserts that shift_pressed works with normal presses"""
        for key in (
                pynput.keyboard.Key.shift,
                pynput.keyboard.Key.shift_l,
                pynput.keyboard.Key.shift_r):
            self.controller.press(key)
            self.assertTrue(
                self.controller.shift_pressed,
                'shift_pressed was not set with %s down' % key.name)
            self.controller.release(key)
            self.assertFalse(
                self.controller.shift_pressed,
                'shift_pressed was incorrectly set')

    def test_shift_pressed_caps_lock(self):
        """Asserts that shift_pressed is True when caps lock is toggled"""
        self.controller.tap(pynput.keyboard.Key.caps_lock)
        self.assertTrue(
            self.controller.shift_pressed,
            'shift_pressed was not set with caps lock toggled')

        self.controller.tap(pynput.keyboard.Key.caps_lock)
        self.assertFalse(
            self.controller.shift_pressed,
            'shift_pressed was not deactivated with caps lock toggled')

    def test_pressed_shift(self):
        """Asserts that pressing and releasing a Latin character while pressing
        shift causes it to shift to upper case"""
        with self.capture() as collect:
            with self.controller.pressed(pynput.keyboard.Key.shift):
                self.controller.tap(u'a')

                with self.controller.modifiers as modifiers:
                    self.assertIn(
                        pynput.keyboard.Key.shift,
                        modifiers)

        self.assertIn(
            u'A',
            collect(),
            'shift+a did not yield "A"')

    def test_pressed_is_release(self):
        """Asserts that pressed actually releases the key"""
        with self.capture() as collect:
            with self.controller.pressed(pynput.keyboard.Key.shift):
                self.controller.tap(u'a')

            self.controller.tap(u'a')

            with self.controller.pressed(pynput.keyboard.Key.shift):
                self.controller.tap(u'a')


        self.assertIn(
            u'AaA',
            collect(),
            'Keys were not properly released')

    def test_type_latin(self):
        """Asserts that type works for a Latin string"""
        self.assert_input(
            'Failed to type latin string',
            u'Hello World')

    def test_type_ascii(self):
        """Asserts that type works for an ascii string"""
        self.assert_input(
            'Failed to type ascii string',
            u'abc123, "quoted!"')

    def test_type_nonascii(self):
        """Asserts that type works for a non-ascii strings"""
        self.assert_input(
            'Failed to type Spanish string',
            u'Teclado (informática)')
        self.assert_input(
            'Failed to type Russian string',
            u'Компьютерная клавиатура')

    def test_type_control_codes(self):
        """Asserts that type works for a string containing control codes"""
        self.assert_input(
            'Failed to type latin string',
            u'Hello\tworld')

    def test_controller_events(self):
        """Tests that events sent by a controller are received correctly"""
        with self.assert_event(
                'Failed to send press',
                on_press=lambda k: getattr(k, 'char', None) == u'a'):
            self.controller.press(u'a')
        with self.assert_event(
                'Failed to send release',
                on_release=lambda k: getattr(k, 'char', None) == u'a'):
            self.controller.release(u'a')

        self.controller.tap(pynput.keyboard.Key.enter)
        input()



# coding=utf-8
# pystray
# Copyright (C) 2015-2024 Moses Palmér
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import contextlib
import functools
import time
import unittest

from six.moves import input

import pynput


#: The name of the current backend
BACKEND = pynput.keyboard.Controller.__module__.rsplit('.', 1)[-1][1:]


def _backend(name, f):
    """Returns ``f`` if the current backend is ``name``.

    :param str name: The name of the backend.

    :param f: A value.

    :return: ``f`` or ``None``
    """
    return f if name == BACKEND else None


def notify(message, delay=None, columns=50):
    """Prints a notification on screen.

    :param str message: The message to display.

    :param delay: An optional delay, in seconds, before returning from this
        function
    :type delay: float or None

    :param int columns: The number of columns for the notification.
    """
    # The maximum length of a message line; we need four columns for the
    # frame
    max_length = columns - 4

    # Split the message into lines containing at most max_length characters
    lines = []
    for line in message.splitlines():
        if lines:
            lines.append('')
        for word in line.split():
            if not lines or not lines[-1] \
                    or len(lines[-1]) + 1 + len(word) > max_length:
                lines.append(word)
            else:
                lines[-1] += ' ' + word

    # Print the message
    print('')
    print('+' + '=' * (columns - 2) + '+')
    for line in lines:
        print(('| {:<%ds} |' % max_length).format(line))
    print('+' + '-' * (columns - 2) + '+')

    if delay:
        time.sleep(delay)



#: A decorator to make a test run only on macOS
darwin = functools.partial(_backend, 'darwin')

#: A decorator to make a test run only on Windows
win32 = functools.partial(_backend, 'win32')

#: A decorator to make a test run only on Linux
xorg = functools.partial(_backend, 'xorg')


class EventTest(unittest.TestCase):
    #: The message displayed when this test suite is started
    NOTIFICATION = None

    #: The controller class; if this is defined, :attr:`controller` will be
    # instantiated for every test
    CONTROLLER_CLASS = None

    #: The listener class; this must be defined for subclasses
    LISTENER_CLASS = None

    #: The maximum number of seconds to wait before failing in
    #: :meth:`assert_stop`
    STOP_MAX_WAIT = 3.0

    #: The minimum number of events to accumulate before checking for changes
    #: in :meth:`assertChange`
    CHANGE_MIN_EVENTS = 50

    @classmethod
    def setUpClass(self):
        self.notify(self.NOTIFICATION, 4)
        self.listeners = []

    @classmethod
    def tearDownClass(self):
        remaining = [
            listener
            for listener in self.listeners
            if not (listener.join(0.5) or listener.is_alive)]
        for listener in remaining:
            listener.join()

    def setUp(self):
        if self.CONTROLLER_CLASS is not None:
            self.controller = self.CONTROLLER_CLASS()
        self.suppress = False

    @classmethod
    def notify(self, message, delay=None, columns=50):
        notify(message, delay, columns)

    def listener(self, *args, **kwargs):
        """Creates a listener.

        All arguments are passed to the constructor.
        """
        listener = self.LISTENER_CLASS(suppress=self.suppress, *args, **kwargs)
        self.listeners.append(listener)
        return listener

    @contextlib.contextmanager
    def assert_event(self, failure_message, **kwargs):
        """Asserts that a specific event is emitted when a code block is
        executed.

        :param str failure_message: The message to display upon failure.

        :param args: Arguments to pass to the listener constructor.

        :param kwargs: Arguments to pass to the listener constructor.
        """
        def wrapper(name, callback):
            def inner(*a):
                if callback(*a):
                    listener.success = True
                    return False

            return inner if callback else None

        with self.listener(**{
                name: wrapper(name, callback)
                for name, callback in kwargs.items()}) as listener:
            time.sleep(0.1)
            listener.success = False
            yield

            for _ in range(30):
                time.sleep(0.1)
                if listener.success:
                    break

        self.assertTrue(
            listener.success,
            failure_message)

    def assert_stop(self, failure_message, **callbacks):
        """Asserts that a listener stop within :attr:`STOP_MAX_WAIT` seconds.

        :param str failure_message: The message to display upon failure.

        :param args: Arguments to pass to the listener constructor.

        :param callbacks: The callbacks for checking whether change has
            occurred.
        """
        success = False
        listener = self.listener(**callbacks)
        with listener:
            for _ in range(10):
                time.sleep(self.STOP_MAX_WAIT * 0.1)
                if not listener.running:
                    success = True
                    break

        self.assertTrue(
            success,
            failure_message)

    def assert_cumulative(self, failure_message, **callbacks):
        """Asserts that the callback returns true for at least two thirds of
        the elements.

        At least :attr:`CHANGE_MIN_EVENTS` will be examined.

        :param str failure_message: The message to display upon failure.

        :param callbacks: The callbacks for checking whether change has
            occurred.
        """
        # The lists of accumulated events
        events = {
            name: []
            for name in callbacks}

        def wrapper(name, callback):
            def inner(*a):
                cache = events[name]
                cache.append(a)

                total_length = len(cache)
                if total_length > self.CHANGE_MIN_EVENTS:
                    change_length = len([
                        None
                        for i, b in enumerate(cache[1:])
                        if callback(cache[i], b)])

                    if change_length > (2 * total_length) / 3:
                        return False

            return inner if callback else None

        self.assert_stop(failure_message, **{
            name: wrapper(name, callback)
            for name, callback in callbacks.items()})

    def confirm(self, statement, *fmt):
        """Asks the user to confirm a statement.

        :param str statement: The statement to confirm.

        :raises AssertionError: if the user does not confirm
        """
        valid_responses = ('yes', 'y', 'no', 'n')
        accept_responses = valid_responses[:2]

        message = ('\n' + statement % fmt) + ' '
        while True:
            response = input(message)
            if response.lower() in valid_responses:
                self.assertIn(
                    response.lower(), accept_responses,
                    'User declined statement "%s"' % message)
                return
            else:
                print(
                    'Please respond %s' % ', '.join(
                        '"%s"' % r for r in valid_responses))



# -*- coding: utf-8 -*-

import datetime
import sys
import os

# Make sure we can import the package
sys.path.insert(
    0,
    os.path.join(
        os.path.dirname(__file__),
        os.path.pardir,
        'lib'))

os.environ['PYNPUT_BACKEND'] = 'dummy'

import pynput as package
import pynput._info as INFO


# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.viewcode']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = package.__package__

original_year = 2015
current_year = datetime.date.today().year
if original_year == current_year:
    copyright = u'%d, %s' % (
        current_year,
        INFO.__author__)
else:
    copyright = u'%d-%d, %s' % (
        original_year,
        current_year,
        INFO.__author__)

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.

# The short X.Y version.
version = '.'.join(str(v) for v in INFO.__version__)

# The full version, including alpha/beta/rc tags.
release = version

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'alabaster'

# Output file base name for HTML help builder.
htmlhelp_basename = '%sdoc' % package.__package__

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (
        'index',
        package.__package__,
        u'%s Documentation' % package.__package__,
        [package._info.__author__],
        3)]



# coding=utf-8
# pystray
# Copyright (C) 2015-2024 Moses Palmér
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import numbers
import pynput.mouse
import time

from . import EventTest


class MouseControllerTest(EventTest):
    NOTIFICATION = (
        'This test case is non-interactive, so you must not use the mouse.\n'
        'You may need to keep the mouse pointer away from this window to '
        'avoid interference.')
    CONTROLLER_CLASS = pynput.mouse.Controller
    LISTENER_CLASS = pynput.mouse.Listener

    def assert_movement(self, failure_message, d):
        """Asserts that movement results in corresponding change of pointer
        position.

        :param str failure_message: The message to display upon failure.

        :param tuple d: The movement vector.
        """
        pos = self.controller.position
        self.controller.move(*d)
        time.sleep(1)
        self.assertEqual(
            self.controller.position,
            tuple(o + n for o, n in zip(pos, d)),
            failure_message)

    def test_buttons(self):
        """Asserts that all buttons defined for the base mouse interface are
        defined for the current platform"""
        from pynput.mouse._base import Button
        for button in Button:
            self.assertTrue(
                hasattr(pynput.mouse.Button, button.name),
                '%s is not defined for the current platform' % button.name)

    def test_position_get(self):
        """Tests that reading the position returns consistent values"""
        position = self.controller.position

        self.assertTrue(
            all(isinstance(i, numbers.Number) for i in position),
            'Not all coordinates in %s are numbers' % str(position))

        self.assertEqual(
            position,
            self.controller.position,
            'Second read of position returned different value')

    def test_position_set(self):
        """Tests that writing the position updates the position value"""
        position = self.controller.position
        new_position = tuple(i + 1 for i in position)

        self.controller.position = new_position
        time.sleep(1)

        self.assertEqual(
            new_position,
            self.controller.position,
            'Updating position failed')

    def test_position_set_float(self):
        """Tests that writing a floating point position does not crash"""
        position = self.controller.position
        new_position = tuple(i + 1.5 for i in position)

        self.controller.position = new_position

    def test_press(self):
        """Tests that press works"""
        for b in (
                pynput.mouse.Button.left,
                pynput.mouse.Button.right):
            with self.assert_event(
                    'Failed to send press event',
                    on_click=lambda x, y, button, pressed:
                    button == b and pressed):
                self.controller.press(b)
            self.controller.release(b)

    def test_release(self):
        """Tests that release works"""
        for b in (
                pynput.mouse.Button.left,
                pynput.mouse.Button.right):
            self.controller.press(b)
            with self.assert_event(
                    'Failed to send release event',
                    on_click=lambda x, y, button, pressed:
                    button == b and not pressed):
                self.controller.release(b)

    def test_left(self):
        """Tests that moving left works"""
        ox, oy = self.controller.position
        with self.assert_event(
                'Failed to send move left event',
                on_move=lambda x, y: x < ox):
            self.assert_movement(
                'Pointer did not move',
                (-1, 0))

    def test_right(self):
        """Tests that moving right works"""
        ox, oy = self.controller.position
        with self.assert_event(
                'Failed to send move right event',
                on_move=lambda x, y: x > ox):
            self.assert_movement(
                'Pointer did not move',
                (1, 0))

    def test_up(self):
        """Tests that moving up works"""
        ox, oy = self.controller.position
        with self.assert_event(
                'Failed to send move up event',
                on_move=lambda x, y: y < oy):
            self.assert_movement(
                'Pointer did not move',
                (0, -1))

    def test_down(self):
        """Tests that moving down works"""
        ox, oy = self.controller.position
        with self.assert_event(
                'Failed to send move down event',
                on_move=lambda x, y: y > oy):
            self.assert_movement(
                'Pointer did not move',
                (0, 1))

    def test_click(self):
        """Tests that click works"""
        for b in (
                pynput.mouse.Button.left,
                pynput.mouse.Button.right):
            events = [True, False]
            events.reverse()

            def on_click(x, y, button, pressed):
                if button == b:
                    self.assertEqual(
                        pressed,
                        events.pop(),
                        'Unexpected event')
                return len(events) == 0

            with self.assert_event(
                    'Failed to send click events',
                    on_click=on_click):
                self.controller.click(b)

    def test_scroll_up(self):
        """Tests that scrolling up works"""
        with self.assert_event(
                'Failed to send scroll up event',
                on_scroll=lambda x, y, dx, dy: dy > 0):
            self.controller.scroll(0, 1)

    def test_scroll_down(self):
        """Tests that scrolling down works"""
        with self.assert_event(
                'Failed to send scroll down event',
                on_scroll=lambda x, y, dx, dy: dy < 0):
            self.controller.scroll(0, -1)



#!/usr/bin/env python
# coding: utf-8
# Copyright 2015 Moses Palmér
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
"""
Converts <keysymdef.h> to Python mappings.
"""

import datetime
import re
import sys
import unicodedata


#: The regular expression used to extract normal values; they are on the form:
#:
#:     #define XK_<name> <hex keysym> /* <unicode point> <character name> */
KEYSYM_RE = re.compile(r'''(?mx)
    # name
    \#define \s+ XK_([a-zA-Z0-9_]+)\s+

    # keysym
    0x([0-9a-fA-F]+)\s+

    # codepoint
    /\*\s* U\+([0-9a-fA-F]+)\s+.*?\*/

    # description, not present
    ()''')

#: The prefix used for dead keys
DEAD_PREFIX = 'dead_'

#: The regular expression used to extract dead key values; they are on the form:
#:
#:     #define XK_dead_<name> <hex keysym> [/* description */]
DEAD_KEYSYM_RE = re.compile(r'''(?mx)
    # name
    \#define \s+ XK_(%s[a-zA-Z0-9_]+)\s+

    # keysym
    0x([0-9a-fA-F]+)\s*

    # codepoint, not present
    ()

    # description
    (?:/\*(.*?)\*/)?''' % DEAD_PREFIX)

#: The prefix used for keypad keys
KEYPAD_PREFIX = 'KP_'

#: The regular expression used to extract keypad key values; they are on the
#: form:
#:
#:     #define XK_KP_<name> <hex keysym> [/* description */]
KP_KEYPAD_RE = re.compile(r'''(?mx)
    # name
    \#define \s+ XK_(%s[a-zA-Z0-9_]+)\s+

    # keysym
    0x([0-9a-fA-F]+)\s*

    # codepoint, not present
    ()

    # description
    (?:/\*(.*?)\*/)?''' % KEYPAD_PREFIX)


def lookup(name):
    """Looks up a named unicode character.

    If it does not exist, ``None`` is returned, otherwise the code point is
    returned.

    :return: a hex number as a string or ``None``
    """
    try:
        return '%04X' % ord(unicodedata.lookup(name))
    except KeyError:
        return None

# A mapping from dead keys to their unicode codepoints
DEAD_CODEPOINTS = {
    name: (
        lookup('COMBINING ' + codepoint),
        lookup(codepoint))
    for name, codepoint in {
        'abovecomma': 'COMMA ABOVE RIGHT',
        'abovedot': 'DOT ABOVE',
        'abovereversedcomma': 'TURNED COMMA ABOVE',
        'abovering': 'RING ABOVE',
        'aboveverticalline': 'VERTICAL LINE ABOVE',
        'acute': 'ACUTE ACCENT',
        'belowbreve': 'BREVE BELOW',
        'belowcircumflex': 'CIRCUMFLEX ACCENT BELOW',
        'belowcomma': 'COMMA BELOW',
        'belowdiaeresis': 'DIAERESIS BELOW',
        'belowdot': 'DOT BELOW',
        'belowmacron': 'MACRON BELOW',
        'belowring': 'RING BELOW',
        'belowtilde': 'TILDE BELOW',
        'belowverticalline': 'VERTICAL LINE BELOW',
        'breve': 'BREVE',
        'caron': 'CARON',
        'cedilla': 'CEDILLA',
        'circumflex': 'CIRCUMFLEX ACCENT',
        'diaeresis': 'DIAERESIS',
        'doubleacute': 'DOUBLE ACUTE ACCENT',
        'doublegrave': 'DOUBLE GRAVE ACCENT',
        'grave': 'GRAVE ACCENT',
        'hook': 'HOOK ABOVE',
        'horn': 'HORN',
        'invertedbreve': 'INVERTED BREVE BELOW',
        'iota': 'GREEK YPOGEGRAMMENI',
        'longsolidusoverlay': 'LONG SOLIDUS OVERLAY',
        'lowline': 'LOW LINE',
        'macron': 'MACRON',
        'ogonek': 'OGONEK',
        'stroke': 'SHORT STROKE OVERLAY',
        'tilde': 'TILDE'}.items()}


def definitions(data):
    """Yields all keysym as the tuple ``(name, keysym, (first, second))``.

    ``(first, second))`` is a tuple of codepoints. The first value is the one
    to use in the lookup table, and the second one is used only by dead keys to
    indicate the non-combining version. If both are ``None``, the keysym is a
    keypad key.

    If a codepoint is ``None``, the definition is for a dead key with no
    unicode codepoint.
    """
    for line in data:
        for regex in (KEYSYM_RE, DEAD_KEYSYM_RE, KP_KEYPAD_RE):
            m = regex.search(line)
            if m:
                name, keysym, codepoint, description = m.groups()
                if codepoint:
                    # If the code point is specified, this keysym corresponds
                    # to a normal character
                    yield (
                        name,
                        (keysym, (codepoint, codepoint)))
                    break

                elif name.startswith(DEAD_PREFIX) and (
                        not description or 'alias for' not in description):
                    yield (
                        name,
                        (keysym, DEAD_CODEPOINTS.get(
                            name[len(DEAD_PREFIX):],
                            (None, None))))
                    break

                elif name.startswith(KEYPAD_PREFIX):
                    yield (
                        name,
                        (keysym, (None, None)))
                    break


def main():
    syms = sorted(list(definitions(sys.stdin.read().splitlines())))
    sys.stdout.write('''# coding: utf-8
# pynput
# Copyright (C) 2015-2024-%d Moses Palmér
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=C0111,C0302

SYMBOLS = {
%s}

DEAD_KEYS = {
%s}

KEYPAD_KEYS = {
%s}

CHARS = {
    codepoint: name
    for name, (keysym, codepoint) in SYMBOLS.items()
    if codepoint}

KEYSYMS = {
    keysym: name
    for name, (keysym, codepoint) in SYMBOLS.items()
    if codepoint}
''' % (
        datetime.date.today().year,
        ',\n'.join(
            '    \'%s\': (0x%s, %s)' % (
                    name,
                    keysym,
                    'u\'\\u%s\'' % first if first else None)
            for name, (keysym, (first, second)) in syms),
        ',\n'.join(
            '    %s: %s' % (
                    'u\'\\u%s\'' % first,
                    'u\'\\u%s\'' % second)
            for name, (keysym, (first, second)) in syms
            if name.startswith(DEAD_PREFIX)
                and first and second and first != second),
        ',\n'.join(
            '    \'%s\': 0x%s' % (name, keysym)
            for name, (keysym, (first, second)) in syms
            if name.startswith(KEYPAD_PREFIX))))

main()



# coding=utf-8
# pystray
# Copyright (C) 2015-2024 Moses Palmér
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import pynput.mouse
import time

from . import EventTest, darwin, win32, xorg


class MouseListenerTest(EventTest):
    NOTIFICATION = (
        'This test case is interactive, so you must follow the instructions '
        'on screen.\n'
        'You do not have to perform any actions on this specific window.')
    LISTENER_CLASS = pynput.mouse.Listener

    def test_stop(self):
        """Tests that stopping the listener from a different thread works"""
        listener = self.listener()

        listener.start()
        listener.wait()
        self.notify('Move mouse, click button or scroll')
        listener.stop()

        time.sleep(1)

    def test_stop_no_wait(self):
        """Tests that stopping the listener from a different thread without
        waiting for the listener to start works"""
        listener = self.listener()

        listener.start()
        self.notify('Move mouse, click button or scroll')
        listener.stop()

        time.sleep(1)

    def test_move(self):
        """Tests that move events are emitted at all"""
        self.notify('Move mouse pointer')
        self.assert_cumulative(
            'Failed to register movement',
            on_move=lambda a, b: True)

    def test_left(self):
        """Tests that move left events are emitted correctly"""
        self.notify('Move mouse pointer left')
        self.assert_cumulative(
            'Failed to register movement',
            on_move=lambda a, b: b[0] < a[0])

    def test_right(self):
        """Tests that move right events are emitted correctly"""
        self.notify('Move mouse pointer right')
        self.assert_cumulative(
            'Failed to register movement',
            on_move=lambda a, b: b[0] > a[0])

    def test_up(self):
        """Tests that move up events are emitted correctly"""
        self.notify('Move mouse pointer up')
        self.assert_cumulative(
            'Failed to register movement',
            on_move=lambda a, b: b[1] < a[1])

    def test_down(self):
        """Tests that move down events are emitted correctly"""
        self.notify('Move mouse pointer down')
        self.assert_cumulative(
            'Failed to register movement',
            on_move=lambda a, b: b[1] > a[1])

    def test_click_left(self):
        """Tests that left click events are emitted"""
        self.notify('Click left mouse button')
        self.assert_stop(
            'No left click registered',
            on_click=lambda x, y, button, pressed: not (
                pressed and button == pynput.mouse.Button.left))

    def test_click_right(self):
        """Tests that right click events are emitted"""
        self.notify('Click right mouse button')
        self.assert_stop(
            'No right click registered',
            on_click=lambda x, y, button, pressed: not (
                pressed and button == pynput.mouse.Button.right))

    def test_scroll_up(self):
        """Tests that scroll up events are emitted"""
        self.notify('Scroll up')
        self.assert_stop(
            'No scroll up registered',
            on_scroll=lambda x, y, dx, dy: not (
                dy > 0))

    def test_scroll_down(self):
        """Tests that scroll down events are emitted"""
        self.notify('Scroll down')
        self.assert_stop(
            'No scroll down registered',
            on_scroll=lambda x, y, dx, dy: not (
                dy < 0))

    def test_suppress(self):
        """Tests that passing ``suppress`` prevents events from propagating"""
        self.suppress = True
        self.notify(
            'Click right mouse button where it will have an effect, and then '
            'press the left mouse button')
        self.assert_stop(
            'No right click registered',
            on_click=lambda x, y, button, pressed: not (
                pressed and button == pynput.mouse.Button.left))
        self.confirm('Was the action suppressed?')

    def test_reraise(self):
        """Tests that exception are reraised"""
        class MyException(Exception): pass

        def on_click(x, y, button, pressed):
            raise MyException()

        with self.assertRaises(MyException):
            with pynput.mouse.Listener(
                    on_click=on_click) as l:
                self.notify('Click any button')
                l.join()

    @darwin
    def test_options_darwin(self):
        """Tests that options are correctly set on OSX"""
        self.assertTrue(
            pynput.mouse.Listener(
                darwin_test=True,
                win32_test=False,
                xorg_test=False)._options['test'])

    @win32
    def test_options_win32(self):
        """Tests that options are correctly set on Windows"""
        self.assertTrue(
            pynput.mouse.Listener(
                darwin_test=False,
                win32_test=True,
                xorg_test=False)._options['test'])

    @xorg
    def test_options_xorg(self):
        """Tests that options are correctly set on Linux"""
        self.assertTrue(
            pynput.mouse.Listener(
                darwin_test=False,
                win32_test=False,
                xorg_test=True)._options['test'])

    def test_events(self):
        """Tests that events are correctly yielded"""
        from pynput.mouse import Button, Events
        with Events() as events:
            self.notify('Move the mouse')
            for event in events:
                if isinstance(event, Events.Move):
                    break

            self.notify('Press the left mouse button')
            for event in events:
                if isinstance(event, Events.Click) \
                        and event.button == Button.left:
                    break

            self.notify('Press the right mouse button')
            for event in events:
                if isinstance(event, Events.Click) \
                        and event.button == Button.right:
                    break

            self.notify('Scroll the mouse')
            for event in events:
                if isinstance(event, Events.Scroll):
                    break

            self.notify('Do not touch the mouse', delay=2.0)
            self.assertIsNone(events.get(1.0))



import os
import sys
sys.path.append(
    os.path.join(os.path.dirname(__file__), '..', 'lib'))

from pynput import keyboard


def on_press(key):
    print('=> %s' % key)
    if key == keyboard.Key.esc:
        return False


def on_release(key):
    print('=< %s' % key)


with keyboard.Listener(
        on_press=on_press,
        on_release=on_release) as listener:
    print('Press esc to exit')
    listener.join()



# coding=utf-8
# pystray
# Copyright (C) 2015-2024 Moses Palmér
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import unittest

from six.moves import queue

from pynput.keyboard import (
    GlobalHotKeys,
    HotKey,
    Key as k,
    KeyCode as kc,
)

from . import notify


class KeyboardHotKeyTest(unittest.TestCase):
    def test_parse_invalid(self):
        with self.assertRaises(ValueError) as e:
            HotKey.parse('invalid')
        self.assertEqual(e.exception.args, ('invalid',))

        with self.assertRaises(ValueError) as e:
            HotKey.parse('a+')
        self.assertEqual(e.exception.args, ('a+',))

        with self.assertRaises(ValueError) as e:
            HotKey.parse('<ctrl>+<halt>')
        self.assertEqual(e.exception.args, ('<halt>',))

        with self.assertRaises(ValueError) as e:
            HotKey.parse('<ctrl>+a+a')
        self.assertEqual(e.exception.args, ('<ctrl>+a+a',))

        with self.assertRaises(ValueError) as e:
            HotKey.parse('<ctrl>+a+A')
        self.assertEqual(e.exception.args, ('<ctrl>+a+A',))

    def test_parse_valid(self):
        self.assertSequenceEqual(
            HotKey.parse('a'),
            [
                kc.from_char('a')])
        self.assertSequenceEqual(
            HotKey.parse('A'),
            [
                kc.from_char('a')])
        self.assertSequenceEqual(
            HotKey.parse('<ctrl>+a'),
            [
                k.ctrl,
                kc.from_char('a')])
        self.assertSequenceEqual(
            HotKey.parse('<ctrl>+<alt>+a'),
            [
                k.ctrl,
                k.alt,
                kc.from_char('a')])
        self.assertSequenceEqual(
            HotKey.parse('<ctrl>+<123456>'),
            [
                k.ctrl,
                kc.from_vk(123456)])

    def test_activate_single(self):
        activations = []
        def on_activate():
            activations.append(True)

        hk = HotKey({kc.from_char('a')}, on_activate)

        hk.press(kc.from_char('b'))
        self.assertEqual(0, len(activations))
        hk.release(kc.from_char('b'))
        self.assertEqual(0, len(activations))

        hk.press(kc.from_char('a'))
        self.assertEqual(1, len(activations))
        hk.release(kc.from_char('a'))
        self.assertEqual(1, len(activations))

        hk.press(kc.from_char('a'))
        self.assertEqual(2, len(activations))
        hk.press(kc.from_char('a'))
        self.assertEqual(2, len(activations))
        hk.release(kc.from_char('a'))
        self.assertEqual(2, len(activations))

    def test_activate_combo(self):
        activations = []
        def on_activate():
            activations.append(True)

        hk = HotKey({k.ctrl, kc.from_char('a')}, on_activate)

        hk.press(kc.from_char('b'))
        self.assertEqual(0, len(activations))
        hk.release(kc.from_char('b'))
        self.assertEqual(0, len(activations))

        hk.press(kc.from_char('a'))
        self.assertEqual(0, len(activations))
        hk.release(kc.from_char('a'))
        self.assertEqual(0, len(activations))

        hk.press(k.ctrl)
        self.assertEqual(0, len(activations))
        hk.release(k.ctrl)
        self.assertEqual(0, len(activations))

        hk.press(kc.from_char('a'))
        hk.press(k.ctrl)
        self.assertEqual(1, len(activations))
        hk.press(kc.from_char('a'))
        hk.press(k.ctrl)
        self.assertEqual(1, len(activations))
        hk.release(k.ctrl)
        hk.press(k.ctrl)
        self.assertEqual(2, len(activations))
        hk.release(kc.from_char('a'))
        hk.press(kc.from_char('a'))
        self.assertEqual(3, len(activations))

    def test_hotkeys(self):
        q = queue.Queue()

        with GlobalHotKeys({
                '<ctrl>+<shift>+a': lambda: q.put('a'),
                '<ctrl>+<shift>+b': lambda: q.put('b'),
                '<ctrl>+<shift>+c': lambda: q.put('c')}):
            notify('Press <ctrl>+<shift>+a')
            self.assertEqual('a', q.get())

            notify('Press <ctrl>+<shift>+b')
            self.assertEqual('b', q.get())

            notify('Press <ctrl>+<shift>+c')
            self.assertEqual('c', q.get())



#!/usr/bin/env python
# coding: utf-8

import os
import setuptools


#: The name of the package on PyPi
PYPI_PACKAGE_NAME = 'pynput'

#: The name of the main Python package
MAIN_PACKAGE_NAME = 'pynput'

#: The package URL
PACKAGE_URL = 'https://github.com/moses-palmer/pynput'

#: The author email
AUTHOR_EMAIL = 'moses.palmer@gmail.com'

#: The runtime requirements
RUNTIME_PACKAGES = [
    'six']

#: Additional requirements used during setup
SETUP_PACKAGES = [
    'setuptools-lint >=0.5',
    'sphinx >=1.3.1',
    'twine >=4.0']

#: Packages requires for different environments
EXTRA_PACKAGES = {
    ':sys_platform == "darwin"': [
        'pyobjc-framework-ApplicationServices >=8.0',
        'pyobjc-framework-Quartz >=8.0'],
    ':"linux" in sys_platform': [
        'evdev >= 1.3',
        'python-xlib >= 0.17'],
    ':python_version == "2.7"': [
        'enum34']}


# Read globals from ._info without loading it
INFO = {}
with open(os.path.join(
        os.path.dirname(__file__),
        'lib',
        MAIN_PACKAGE_NAME,
        '_info.py'), 'rb') as f:
    for line in f.read().decode('utf-8').splitlines():
        try:
            name, value = (i.strip() for i in line.split('='))
            if name.startswith('__') and name.endswith('__'):
                INFO[name[2:-2]] = eval(value)
        except ValueError:
            pass


# Load the read me
try:
    with open(os.path.join(
            os.path.dirname(__file__),
            'README.rst'), 'rb') as f:
        README = f.read().decode('utf-8')

    with open(os.path.join(
            os.path.dirname(__file__),
            os.path.join('docs', 'mouse-usage.rst')), 'rb') as f:
        README += '\n\n' + f.read().decode('utf-8')

    with open(os.path.join(
            os.path.dirname(__file__),
            os.path.join('docs', 'keyboard-usage.rst')), 'rb') as f:
        README += '\n\n' + f.read().decode('utf-8')
except IOError:
    README = ''


# Load the release notes
try:
    with open(os.path.join(
            os.path.dirname(__file__),
            'CHANGES.rst'), 'rb') as f:
        CHANGES = f.read().decode('utf-8')
except IOError:
    CHANGES = ''


setuptools.setup(
    name=PYPI_PACKAGE_NAME,
    version='.'.join(str(i) for i in INFO['version']),
    description='Monitor and control user input devices',
    long_description=README + '\n\n' + CHANGES,

    install_requires=RUNTIME_PACKAGES,
    setup_requires=RUNTIME_PACKAGES + SETUP_PACKAGES,
    extras_require=EXTRA_PACKAGES,

    author=INFO['author'],
    author_email=AUTHOR_EMAIL,

    url=PACKAGE_URL,

    packages=setuptools.find_packages(
        os.path.join(
            os.path.dirname(__file__),
            'lib')),
    package_dir={'': 'lib'},
    zip_safe=True,

    test_suite='tests',

    license='LGPLv3',
    keywords='control mouse, mouse input, control keyboard, keyboard input',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 '
        '(LGPLv3)',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows :: Windows NT/2000',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Monitoring'])



#!/usr/bin/env python
# coding: utf-8
# Copyright 2016 Moses Palmér
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
"""
Converts virtual key codes from <winuser.h> to Python definitions.
"""

import datetime
import re
import sys
import unicodedata


#: The regular expression used to extract virtual key codes; they are on the
#: form:
#:
#:     #define VK_<name> <hex vk>
VK_RE = re.compile(r'''(?mx)
    # name
    \#define \s+ VK_([a-zA-Z0-9_]+)\s+

    # vk
    0x([0-9a-fA-F]+)\s*

    # Discard rest of line
    .*''')


def definitions(data):
    """Yields all virtual key codes as the tuple ``(name, vk)``.
    """
    for line in data:
        m = VK_RE.search(line)
        if not m:
            continue

        name, vk = m.groups()
        yield (name, int(vk, 16))


def main():
    data = sys.stdin.read().splitlines()
    sys.stdout.write('''# coding: utf-8
# pynput
# Copyright (C) 2015-2024-%d Moses Palmér
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=C0111,C0302

%s
''' % (
        datetime.date.today().year,
        '\n'.join(
            '%s = %d' % (name, vk)
            for name, vk in definitions(data))))

main()



#!/usr/bin/env python

import os
import re
import subprocess
import sys

DESCRIPTION = """Makes a full release.

This script will update the version number of the package and perform all steps
necessary to make a full release.
"""

ROOT = os.path.join(os.path.dirname(__file__), os.pardir)

LIB_DIR = os.path.join(ROOT, 'lib')

PACKAGE_NAME = next(name for name in os.listdir(LIB_DIR) if name[0] != '_')

PACKAGE_DIR = os.path.join(LIB_DIR, PACKAGE_NAME)


def main(version):
    assert_current_branch_is_clean()
    update_info(version)
    try:
        check_readme()
        check_release_notes(version)
        commit_changes(version)
        try:
            tag_release(version)
            try:
                push_to_origin()
                build_packages()
                upload_to_pypi(version)
            except:
                tag_release.undo(version)
                raise
        except:
            commit_changes.undo()
            raise
    except:
        update_info.undo()
        raise


def assert_current_branch_is_clean():
    """Asserts that the current branch contains no local changes.

    :raises RuntimeError: if the repository contains local changes
    """
    try:
        git('diff-index', '--quiet', 'HEAD', '--')
    except RuntimeError as e:
        print(e.args[0] % e.args[1:])
        raise RuntimeError('Your repository contains local changes')


def update_info(version):
    """Updates the version information in ``._info.``

    :param tuple version: The version to set.
    """
    gsub(
        os.path.join(PACKAGE_DIR, '_info.py'),
        re.compile(r'__version__\s*=\s*(\([0-9]+(\s*,\s*[0-9]+)*\))'),
        1,
        repr(version),
    )


def _update_info_undo():
    command('git', 'checkout', ROOT)


update_info.undo = _update_info_undo


def check_readme():
    """Verifies that the ``README`` is *reStructuredText* compliant."""
    python('setup.py', 'check', '--restructuredtext', '--strict')


def check_release_notes(version):
    """Displays the release notes and allows the user to cancel the release
    process.

    :param tuple version: The version that is being released.
    """
    CHANGES = os.path.join(ROOT, 'CHANGES.rst')
    header = 'v%s' % '.'.join(str(v) for v in version)

    # Read the release notes
    found = False
    release_notes = []
    with open(CHANGES) as f:
        for line in (l.strip() for l in f):
            if found:
                if not line:
                    # Break on the first empty line after release notes
                    break
                elif set(line) == {'-'}:
                    # Ignore underline lines
                    continue
                release_notes.append(line)

            elif line.startswith(header):
                # The release notes begin after the header
                found = True

    while True:
        # Display the release notes
        sys.stdout.write('Release notes for %s:\n' % header)
        sys.stdout.write(
            '\n'.join('  %s' % release_note for release_note in release_notes)
            + '\n'
        )
        sys.stdout.write('Is this correct [yes/no]? ')
        sys.stdout.flush()
        response = sys.stdin.readline().strip()
        if response in ('yes', 'y'):
            break
        elif response in ('no', 'n'):
            raise RuntimeError('Release notes are not up to date')


def commit_changes(version):
    """Commits all local changes.

    :param tuple version: The version that is being released.
    """
    git('commit', '-a', '-m', 'Release %s' % '.'.join(str(v) for v in version))


def _commit_changes_undo():
    git('reset', '--hard', 'HEAD^')


commit_changes.undo = _commit_changes_undo


def tag_release(version):
    """Tags the current commit as a release.

    :param version: The version that is being released.
    :type version: tuple of version parts
    """
    git(
        'tag',
        '-a',
        '-m',
        'Release %s' % '.'.join(str(v) for v in version),
        'v' + '.'.join(str(v) for v in version),
    )


def _tag_release_undo(version):
    git('tag', '-d', 'v' + '.'.join(str(v) for v in version))


tag_release.undo = _tag_release_undo


def push_to_origin():
    """Pushes master to origin."""
    print('Pushing to origin...')

    git('push', 'origin', 'HEAD:master')
    git('push', '--tags')


def build_packages():
    """Builds the packages, generating artifacts under ``dist``."""
    print('Building packages...')

    python(os.path.join(ROOT, 'setup.py'), 'sdist', 'bdist_wheel')


def upload_to_pypi(version):
    """Uploads this project to PyPi.

    :param tuple version: The version that is being released.
    """
    print('Uploading to PyPi...')

    python(
        '-m',
        'twine',
        'upload',
        '--skip-existing',
        os.path.join(ROOT, 'dist', '*'),
    )


def git(*args):
    """Executes ``git`` with the command line arguments given.

    :param args: The arguments to ``git``.

    :return: stdout of ``git``

    :raises RuntimeError: if ``git`` returns non-zero
    """
    return command('git', *args)


def python(*args):
    """Executes *Python* with the command line arguments given.

    The *Python* used is the one executing the current script.

    :param args: The arguments to *Python*.

    :return: stdout of *Python*

    :raises RuntimeError: if *Python* returns non-zero
    """
    return command(sys.executable, *args)


def gsub(path, regex, group, replacement):
    """Runs a regular expression on the contents of a file and replaces a
    group.

    :param str path: The path to the file.

    :param regex: The regular expression to use.

    :param int group: The group of the regular expression to replace.

    :param str replacement: The replacement string.
    """
    with open(path) as f:
        data = f.read()

    def sub(match):
        full = match.group(0)
        o = match.start(0)
        return (
            full[: match.start(group) - o]
            + replacement
            + full[match.end(group) - o :]
        )

    with open(path, 'w') as f:
        f.write(regex.sub(sub, data))


def command(*args):
    """Executes a command.

    :param args: The command and arguments.

    :return: stdout of the command

    :raises RuntimeError: if the command returns non-zero
    """
    g = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    stdout, stderr = g.communicate()
    if g.returncode != 0:
        raise RuntimeError(
            'Failed to execute <%s> (%d): %s',
            ' '.join(args),
            g.returncode,
            stdout.decode('utf-8') + '\n\n' + stderr.decode('utf-8'),
        )
    else:
        return stdout.decode('utf-8')


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser(description=DESCRIPTION)

    parser.add_argument(
        'version', type=lambda s: tuple(int(v) for v in s.split('.'))
    )

    try:
        main(**vars(parser.parse_args()))
    except Exception as e:
        try:
            sys.stderr.write(e.args[0] % e.args[1:] + '\n')
        except:
            sys.stderr.write('%s\n' % str(e))
        sys.exit(1)



# coding=utf-8
# pystray
# Copyright (C) 2015-2024 Moses Palmér
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import contextlib
import six
import six.moves.queue as queue
import time
import threading

import pynput.keyboard

from . import EventTest, darwin, win32, xorg

from six.moves import input


class KeyboardListenerTest(EventTest):
    NOTIFICATION = (
        'This test case is interactive, so you must follow the instructions '
        'on screen')
    LISTENER_CLASS = pynput.keyboard.Listener

    @contextlib.contextmanager
    def events(self, timeout=5.0):
        """A context manager that yields keyboard events.

        The context object is a generator that yields the actual events. An
        event is the tuple ``(key, pressed)``.

        The generator is stopped whenever the timeout is triggered. If this
        happens, ``None`` will be yielded as the last event.

        :param timeout: The number of seconds to wait for a key event.
        """
        def generator(q):
            while True:
                try:
                    yield q.get(timeout=timeout)
                except queue.Empty:
                    yield None
                    break

        # Yield the generator and allow the client to capture events
        q = queue.Queue()
        with self.listener(
                on_press=lambda k: q.put((k, True)),
                on_release=lambda k: q.put((k, False))):
            yield generator(q)

    def assert_keys(self, failure_message, *args):
        """Asserts that the list of key events is emitted.

        :param str failure_message: The message to display upon failure.

        :param expected: A sequence of the tuple ``(key, pressed)``, where
            ``key`` is the key and ``pressed`` whether it was pressed.

            ``key`` may be either a string, a :class:`~pynput.keyboard.Key` or
            a :class:`~pynput.keyboard.KeyCode`.

            The tuple may only be a tuple of tuples, in which case any of the
            values will be accepted.
        """
        def normalize(event):
            if not isinstance(event[0], tuple):
                return normalize(((event[0],), event[1]))
            key, is_pressed = event
            return (
                tuple(
                    pynput.keyboard.KeyCode.from_char(key)
                    if isinstance(key, six.string_types)
                    else key.value if key in pynput.keyboard.Key
                    else key
                    for key in event[0]),
                is_pressed)

        original_expected = [normalize(arg) for arg in args]
        remaining = list(original_expected)

        time.sleep(1)

        actual = []
        try:
            with self.events() as events:
                for event in events:
                    if event is None:
                        break

                    expected = remaining.pop(0)
                    current = normalize(event)
                    actual.append(current)
                    self.assertIn(
                        current[0][0],
                        expected[0],
                        '%s was not found in %s' % (
                            expected[0],
                            current[0][0]))
                    self.assertEqual(
                        current[1],
                        expected[1],
                        'Pressed state for %s was incorrect' % (
                            str(current[0][0])))

                    if not remaining:
                        break

            # Ensure that no keys remain
            self.assertSequenceEqual(
                [],
                remaining,
                '%s ([%s] != [%s])' % (
                    failure_message,
                    ' '.join(str(e) for e in original_expected),
                    ' '.join(str(a) for a in actual)))

        finally:
            self.notify('Press <enter> to continue...', delay=0)
            result = input()
            time.sleep(1)
            return result

    def string_to_events(self, s):
        """Yields all events necessary to type a string.

        :param str s: The string.
        """
        for c in s:
            yield (c, True)
            yield (c, False)

    def test_tap(self):
        """Tests that a single key can be tapped"""
        self.notify('Press and release "a"')
        self.assert_keys(
            'Failed to register event',
            ('a', True), ('a', False))

    def test_enter(self):
        """Tests that the enter key can be tapped"""
        self.notify('Press <enter>')
        self.assert_keys(
            'Failed to register event',
            (pynput.keyboard.Key.enter, True))

    def test_modifier(self):
        """Tests that the modifier keys can be tapped"""
        from pynput.keyboard import Key
        for key in (
                (Key.alt, Key.alt_l, Key.alt_r),
                (Key.ctrl, Key.ctrl_l, Key.ctrl_r),
                (Key.shift, Key.shift_l, Key.shift_r)):
            self.notify('Press <%s>' % key[0].name)
            self.assert_keys(
                'Failed to register event',
                (key, True))

    def test_order(self):
        """Tests that the order of key events is correct"""
        self.notify('Type "hello world"')
        self.assert_keys(
            'Failed to register event',
            *tuple(self.string_to_events('hello world')))

    def test_shift(self):
        """Tests that <shift> yields capital letters"""
        self.notify('Type "TEST" with <shift> pressed')
        self.assert_keys(
            'Failed to register event',
            (
                (
                    pynput.keyboard.Key.shift,
                    pynput.keyboard.Key.shift_l,
                    pynput.keyboard.Key.shift_r),
                True),
            *tuple(self.string_to_events('TEST')))

    def test_modifier_and_normal(self):
        """Tests that the modifier keys do not stick"""
        from pynput.keyboard import Key
        self.notify('Press a, <ctrl>, a')
        self.assert_keys(
            'Failed to register event',
            ('a', True),
            ('a', False),
            ((Key.ctrl, Key.ctrl_l, Key.ctrl_r), True),
            ((Key.ctrl, Key.ctrl_l, Key.ctrl_r), False),
            ('a', True),
            ('a', False))

    def test_suppress(self):
        """Tests that passing ``suppress`` prevents events from propagating"""
        self.suppress = True
        self.notify('Type "hello world"')
        self.assertEqual(
            '',
            self.assert_keys(
                'Failed to register event',
                *tuple(self.string_to_events('hello world'))).strip())

    def test_reraise(self):
        """Tests that exception are reraised"""
        class MyException(Exception): pass

        def on_press(key):
            raise MyException()

        with self.assertRaises(MyException):
            with pynput.keyboard.Listener(
                    on_press=on_press) as l:
                self.notify('Press any key')
                l.join()

    def test_stop(self):
        """Tests that stop works from a separate thread"""
        self.notify('Do not touch the keyboard')

        with pynput.keyboard.Listener() as l:
            def runner():
                time.sleep(1)
                l.stop()

            threading.Thread(target=runner).start()
            l.join(2.0)
            self.assertFalse(
                l.is_alive(),
                'Listener did not stop')

    @darwin
    def test_options_darwin(self):
        """Tests that options are correctly set on OSX"""
        self.assertTrue(
            pynput.keyboard.Listener(
                darwin_test=True,
                win32_test=False,
                xorg_test=False)._options['test'])

    @win32
    def test_options_win32(self):
        """Tests that options are correctly set on Windows"""
        self.assertTrue(
            pynput.keyboard.Listener(
                darwin_test=False,
                win32_test=True,
                xorg_test=False)._options['test'])

    @xorg
    def test_options_xorg(self):
        """Tests that options are correctly set on Linux"""
        self.assertTrue(
            pynput.keyboard.Listener(
                darwin_test=False,
                win32_test=False,
                xorg_test=True)._options['test'])

    def test_events(self):
        """Tests that events are correctly yielded"""
        from pynput.keyboard import Key, KeyCode, Events
        self.notify('Press a, b, a, <esc>')

        with Events() as events:
            result = []
            for event in events:
                if event.key == Key.esc:
                    break
                else:
                    result.append(event)

        self.assertSequenceEqual(
            result,
            [
                Events.Press(KeyCode.from_char('a')),
                Events.Release(KeyCode.from_char('a')),
                Events.Press(KeyCode.from_char('b')),
                Events.Release(KeyCode.from_char('b')),
                Events.Press(KeyCode.from_char('a')),
                Events.Release(KeyCode.from_char('a')),
            ])

        self.notify('Do not touch the keyboard', delay=2.0)
        with Events() as events:
            self.assertIsNone(events.get(1.0))
